/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jbi.descriptor.original;

import java.io.IOException;

import javax.xml.bind.JAXBException;

import org.ow2.petals.jbi.descriptor.AbstractJBIDescriptorBuilder;
import org.ow2.petals.jbi.descriptor.JBIDescriptorException;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.xml.sax.SAXException;

/**
 * This Class defines the JBI Descriptor builder for the kernel.
 * <ul>
 * <li>It can validate JBI descriptor in XML form.</li>
 * <li>It can build class bindings from a XML form.</li>
 * <li>It can build the XML binding from classes form.</li>
 * </ul>
 * <p>
 * Note that this Builder is based on the original JBI schema; it doesn't control the JBI extension parts.
 * </p>
 * <p>
 * This class is thread-safe.
 * </p>
 * 
 * @author Roland NAUDIN - EBM WebSourcing
 */
public class JBIDescriptorBuilder extends AbstractJBIDescriptorBuilder<Jbi> {

    /**
     * JBI schema name
     */
    public static final String XSD_JBI = "jbi.xsd";

    private static JBIDescriptorException INSTANCE_EXCEPTION;

    private static JBIDescriptorBuilder INSTANCE;

    public static synchronized JBIDescriptorBuilder getInstance() throws JBIDescriptorException {
        if (INSTANCE_EXCEPTION != null) {
            throw INSTANCE_EXCEPTION;
        }
        if (INSTANCE == null) {
            try {
                INSTANCE = new JBIDescriptorBuilder();
            } catch (JBIDescriptorException e) {
                INSTANCE_EXCEPTION = e;
                throw INSTANCE_EXCEPTION;
            }
        }
        return INSTANCE;
    }

    protected JBIDescriptorBuilder() throws JBIDescriptorException {
        super(Jbi.class);
        try {
            super.init(Jbi.class, XSD_JBI);
        } catch (final IOException e) {
            throw new JBIDescriptorException("Failed to get resource '" + XSD_JBI + "' from the current class-path", e);
        } catch (final SAXException e) {
            throw new JBIDescriptorException("Failed to parse resource '" + XSD_JBI + "'", e);
        } catch (JAXBException e) {
            throw new JBIDescriptorException("Failed to create JAXB objects", e);
        }
    }


}
